//
// Name: MFACE.PRG
// Project: Fischer Techniek interfacing
// Description: Multiface (von Knobloch GmbH) interface driver
// Language: xBase++
// DLLs : needs inpout32.dll, see io.prg(io.obj)
// Author: Rene Nooteboom
// Email: r.nooteboom2@chello.nl
// Date created: 21-08-2005
// Date updated: 02/09/2006 22:12
// Remark : can control 4 multifaces at the same time
//
//  default values :
//
//  parallel port  I/O   IRQ
//
//   LPT1         378H    7
//   LPT2         278H    5
//   LPT3         3BCH    -
//   LPT4         2BCH    -
//
//  example mface.ini file :  //
//                            LPT3_IO = h378 : hex
//                            LPT3_IO = A99 : assumes hex
//                            LPT3_DELAY = 4
//                            LPT2_IO = 632 : assumes decimal
//                            LPT1_DELAY = 2 // output loops delay
//                            //
//
// FUNCTION MfHexIO(nLptPort) RETURN cHexLptIOAddress 
// FUNCTION MfInit(nLptPort,laskvalues,xIOAddress,nLptDelay) RETURN nLptPort
// FUNCTION MfInput(nLptPort) RETURN nByte
// FUNCTION MfSensor(nLptPort,nE1toE8) RETURN lSensor
// PROCEDURE MfButton(nLptPort,nE1toE8,iSleepIdleTime)
// PROCEDURE MfButtonPushed(nLptPort,nE1toE8,iSleepIdleTime)
// PROCEDURE MfClose(nLptPort)
// Procedure MfCloseAll // also unloads inpout32.dll see io.prg
// PROCEDURE MfMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
// PROCEDURE MfMotorLeft(nLptPort,nMotorNr)
// PROCEDURE MfMotorOff(nLptPort,nMotorNr)
// PROCEDURE MfMotorOn(nLptPort,nMotorNr)
// PROCEDURE MfMotorRight(nLptPort,nMotorNr)
// PROCEDURE MfMotorsOff(nLptPort)
// PROCEDURE MfOffNr(nLptPort,nO1toO8)  // Ausgang # sturen
// PROCEDURE MfOnNr(nLptPort,nO1toO8) // Ausgang # sturen
// PROCEDURE MfOutput(nLptPort,nByte)
// Procedure MfShowUsedIF()
// 
#include "common.ch"
#include "xbp.ch"
#include "noot.ch"
#include "ft.ch"
//
STATIC cHexLptIO_ := {"378","278","3BC","2BC"} // max 4 lptports
Static nLptBasisReg_ := {888,632,956,700}
STATIC nLptStatusReg_ := {889,633,957,701} // offset 1 from dataregister
STATIC nLptControlReg_ := {890,634,958,702} // offset 2 from dataregister
STATIC lLptUsed_ := {FALSE,FALSE,FALSE,FALSE},nMotorByte_:={0,0,0,0}
Static nLptDelay_ := {1,1,1,1}
//
FUNCTION MfInit(nLptPort,laskvalues,xIOAddress,nLptDelay)
//
// all arguments optional,xIOAddress can be numeric decimal or a hex-string
//
 LOCAL GetList:={},nBasisRegIO,nStatusRegIO,nControlRegIO,oCrt, ;
  oParent,i,oOwner
 DEFAULT nLptPort TO 1,laskvalues TO TRUE
 IF xIOAddress == NIL
  xIOAddress := MfGetIOAddress(nLptPort)
 ENDIF
 IF IS_NUM(xIOAddress) // convert to hex
  xIOAddress := padl(alltrim(ntoc(xIOAddress,16,3)),3,"0")
 EndIf
 If empty(nLptDelay)
  nLptDelay := MfGetLptDelay(nLptPort)
 EndIf
 IF laskvalues
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf 
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},7,38,"Select LPT-port")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  Set Cursor on
  @ 2,6 SAY "LPT-port(1 to 4)  :"
  @ 3,6 SAY "IO-Address(in hex):"
  @ 4,6 SAY "Lpt-delay(loops)  :"
  @ 2,30 GET nLptPort PICTURE "9" VALID IS_BETW(nLptPort,1,4)
  @ 3,30 GET xIOAddress PICTURE "@! XXX" VALID IS_HEX(xIOAddress)
  @ 4,30 Get nLptDelay PICTURE "999" VALID nLptDelay > 0
  Read
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf
 EndIf
 IF nLptPort > 0
  If nLptPort > 4
   ALARMTONE
   MsgBox("MfInit():Error: LPT<x> must be 1-4")
   Return -1
  EndIf
  If lLptUsed_[nLptPort]
   ALARMTONE
   MsgBox("MfInit():Error: LPT" + Str(nLptPort,1,0) + " already in use")
   Return -1
  EndIf
  xIOAddress := zeropad(AllTrim(xIOAddress),3)
  nBasisRegIO := cton(xIOAddress,16)
  nStatusRegIO := nBasisRegIO + 1
  nControlRegIO := nBasisRegIO + 2
  If GetIODllHandle() == 0
   LoadIODll()
  EndIf
  If !(GetIODllHandle() == 0)
   cHexLptIO_[nLptPort] := xIOAddress
   nLptBasisReg_[nLptPort] := nBasisRegIO
   nLptStatusReg_[nLptPort] := nStatusRegIO
   nLptControlReg_[nLptPort] := nControlRegIO
   If !empty(nLptDelay)
    nLptDelay_[nLptPort] := nLptDelay
   EndIf
   For i:=1 to nLptDelay_[nLptPort]  
    outbyte(nControlRegIO,142) // initialize interface : h8E=142   
   Next i   
   lLptUsed_[nLptPort] := TRUE
   MfOutput(nLptPort,0) // switch all motors off
  Else
   nLptPort := -1
   ALARMTONE
   MsgBox("MfInit():Error: could not get IO dllhandle")
  EndIf
 Else
  nLptPort := -1
 EndIf
RETURN nLptPort
//
PROCEDURE MfClose(nLptPort)
 LOCAL i
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  MfOutput(nLptPort,0)
  For i:=1 to nLptDelay_[nLptPort]
   outbyte(nLptControlReg_[nLptPort],140) // 140d=1000 1100 = h8C
  Next i
  lLptUsed_[nLptPort] := FALSE
 ENDIF
Return
//
Procedure MfCloseAll()
 LOCAL i
 For i:=1 to 4
  MfClose(i)
 Next i
 CloseIODll()
 Return 
//
FUNCTION MfInput(nLptPort)
 LOCAL nbyte1,nbyte2,ntmp1,ntmp2,nbyte,i
 DEFAULT nLptPort TO 1
 For i:=1 to nLptDelay_[nLptPort] 
  outbyte(nLptControlReg_[nLptPort],142) // 142d=1000 1110=h8E
 Next i
 nbyte1 := inbyte(nLptStatusReg_[nLptPort])
 If !(nbyte1 == -1)
  nbyte1 := numand(nbyte1,"E8") // 1110 1000
  nbyte1 := numxor(nbyte1,"80") // 1000 0000
  ntmp1 := numand(nbyte1,"E0") // 1110 0000
  ntmp2 := numand(nbyte1,"08") * 2  // 0000 1000 and SHL 1
  nbyte1 := numor(ntmp1,ntmp2)
  For i:=1 to nLptDelay_[nLptPort]
   outbyte(nLptControlReg_[nLptPort],143) // 143d=1000 1111=h8F
  Next i
  nbyte2 := inbyte(nLptStatusReg_[nLptPort])
  IF !(nbyte2 == -1)
   nbyte2 := numand(nbyte2,"E8") // 1110 1000
   nbyte2 := numxor(nbyte2,"80") // 1000 0000
   ntmp1 := numand(nbyte2,"E0") // 1110 0000
   ntmp2 := numand(nbyte2,"08") * 2  // 0000 1000 and SHL 1
   nbyte2 := numor(ntmp1,ntmp2) / 16 // SHR 4
   nbyte := numor(nbyte1,nbyte2)
  ELSE
   nbyte := -1
   ALARMTONE
   MsgBox("MfInput():Error: reading port: " + ntoc(nLptStatusReg_[nLptPort],16,3) + "h")
  EndIf
 Else
  nbyte := -1
  ALARMTONE
  MsgBox("MfInput():Error: reading port: " + ntoc(nLptStatusReg_[nLptPort],16,3) + "h")
 EndIf
 For i:=1 to nLptDelay_[nLptPort]
  outbyte(nLptControlReg_[nLptPort],140) // 140d=1000 1100=h8C
 Next i
RETURN nbyte
//
PROCEDURE MfOutput(nLptPort,nByte)
 LOCAL i
 Default nLptport TO 1
 For i:=1 to nLptDelay_[nLptPort]
  outbyte(nLptControlReg_[nLptPort],142) // 142d=1000 1110=h8E initialize
 Next i
 For i:=1 to nLptDelay_[nLptPort]
  outbyte(nLptBasisReg_[nLptPort],nByte)
 Next i
 For i:=1 to nLptDelay_[nLptPort]
  outbyte(nLptControlReg_[nLptPort],140) // 140d=1000 1100=h8C
 Next i
 nMotorByte_[nLptPort] := nByte
Return
//
FUNCTION MfSensor(nLptPort,nE1toE8)
 LOCAL nInput:=0,lSensor:=FALSE
 nInput := MfInput(nLptPort)
 lSensor := isbit(nInput,nE1toE8)
RETURN lSensor
//
PROCEDURE MfMotorsOff(nLptPort)
 MfOutput(nLptPort,0)
RETURN
//
PROCEDURE MfMotor(nLptPort,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nOutputByte
 DEFAULT nLptPort TO 1
 nOutputByte := nMotorByte_[nLptPort]
 DO CASE
  CASE nMotorNr == 1
   nOutputByte := numand(nOutputByte,"FC") // 1111 1100
   nOutputByte := numor(nOutputByte,nDirection) // no shift
  CASE nMotorNr == 2
   nOutputByte := numand(nOutputByte,"F3") // 1111 0011
   nOutputByte := numor(nOutputByte,nDirection*4) // left shift 2 bits
  CASE nMotorNr == 3
   nOutputByte := numand(nOutputByte,"CF") // 1100 1111
   nOutputByte := numor(nOutputByte,nDirection*16) // left shift 4 bits
  CASE nMotorNr == 4
   nOutputByte := numand(nOutputByte,"3F") // 0011 1111
   nOutputByte := numor(nOutputByte,nDirection*64) // left shift 6 bits
  OTHERWISE
 ENDCASE
 MfOutput(nLptPort,nOutputByte)
RETURN
//
PROCEDURE MfButton(nLptPort,nE1toE8,iSleepIdleTime)
 Default iSleepIdleTime to 1  // 1/100s
 If iSleepIdleTime > 0
  DO WHILE !MfSensor(nLptPort,nE1toE8)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !MfSensor(nLptPort,nE1toE8)
  EndDo	
 EndIf
RETURN
//
PROCEDURE MfButtonPushed(nLptPort,nE1toE8,iSleepIdleTime)
 Default iSleepIdleTime to 1  // 1/100s
 If iSleepIdleTime > 0
  DO WHILE MfSensor(nLptPort,nE1toE8)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  Do WHILE MfSensor(nLptPort,nE1toE8)
  EndDo		
 EndIf 	
RETURN
//
PROCEDURE MfOnNr(nLptPort,nO1toO8) // Ausgang # sturen
 LOCAL nByte := nMotorByte_[nLptPort]
 DEFAULT nLptPort TO 1
 nByte := setbit(nByte,nO1toO8)
 MfOutput(nLptPort,nByte)
RETURN
//
PROCEDURE MfOffNr(nLptPort,nO1toO8)  // Ausgang # sturen
 LOCAL nByte := nMotorByte_[nLptPort]
 DEFAULT nLptPort TO 1
 nByte := clearbit(nByte,nO1toO8)
 MfOutput(nLptPort,nByte)
RETURN
//
PROCEDURE MfMotorLeft(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MCCW)
RETURN
//
PROCEDURE MfMotorRight(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MCW)
RETURN
//
PROCEDURE MfMotorOff(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MOFF)
RETURN
//
PROCEDURE MfMotorOn(nLptPort,nMotorNr)
 MfMotor(nLptPort,nMotorNr,MON)
RETURN
//
STATIC FUNCTION MfGetIOAddress(nLptPort)
 LOCAL xIOAddress:=NIL,csearch
 DEFAULT nLptPort TO 1
 IF file("mface.ini")
  readinival("mface.ini")
  csearch := "LPT" + str(nLptPort,1,0) + "_IO" // example: LPT3_IO = h3BC
  IF ini_exist(csearch)
   IF !empty(xIOAddress := ini_value(csearch))
    DO CASE
     CASE IS_NUM(xIOAddress)  // assume decimal number
     CASE IS_CHAR(xIOAddress)
      IF upper(left(xIOAddress,1)) == "H" // hex number follows "Hxxx"
       xIOAddress := padl(alltrim(substr(xIOAddress,2)),3,"0")
      ELSE
       IF !IS_HEX(xIOAddress) // als eerste letter geen ABCDEF
        xIOAddress := NIL
       ELSE
        xIOAddress := padl(alltrim(xIOAddress),3,"0")
       ENDIF
      ENDIF
     OTHERWISE
      xIOAddress := NIL
    ENDCASE
   ENDIF
  ENDIF
  clearinival()
 ENDIF
 IF empty(xIOAddress)
  DO CASE
   CASE nLptPort == 1
    xIOAddress := "378"
   CASE nLptPort == 2
    xIOAddress := "278"
   CASE nLptPort == 3
    xIOAddress := "3BC"
   CASE nLptPort == 4
    xIOAddress := "2BC"
   OTHERWISE
    xIOAddress := "378"
  ENDCASE
 ENDIF
RETURN xIOAddress
//
Static Function MfGetLptDelay(nLptPort)
 LOCAL cSearch,nLptDelay:=1
 Default nLptPort to 1
 If File("mface.ini")
  readinival("mface.ini")
  cSearch := "LPT" + Str(nLptPort,1,0) + "_DELAY"
  If empty(nLptDelay := ini_value(cSearch))
   nLptDelay := 1
  EndIf
 EndIf
Return nLptDelay
//
Procedure MfShowUsedIF()
 LOCAL cmsg:="Multiface uses:",i
 FOR i := 1 TO 4
  IF lLptUsed_[i]
   cmsg += " LPT" + str(i,1,0) + " ,IO:" + MfHexIO(i) + "h"
  ENDIF
 Next i
 MsgBox(cmsg)
RETURN
//
FUNCTION MfHexIO(nLptPort)
 LOCAL cHexIO:="---"
 DEFAULT nLptPort TO 1
 IF nLptPort > 0 .AND. lLptUsed_[nLptPort]
  cHexIO := cHexLptIO_[nLptPort]
 ENDIF
RETURN cHexIO
//
// einde mface.prg
//
